package ruon.jiao;
/**
 * This API is the property of R-U-ON LLC
 * You can embed this API in other applications and freely distribute it.
 * R-U-ON maintains the right to block agents that create an extreme load
 * on the server.
 */

import java.util.Date;
import java.text.MessageFormat;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.*;
import java.net.*;

class IAOP {

    final private static String url = "https://agent.r-u-on.com/iaop1";

    private int iter = 0;
    private long created = System.currentTimeMillis();
    String host;
    String ip;
    private String pver = "1";
    String os;

    IAOP() {
        try {
            InetAddress inet = InetAddress.getLocalHost();
            host = inet.getHostName();
            ip = inet.getHostAddress(); // Don't really like this but URLConnection won't tell me...
        } catch (UnknownHostException e) {
            host = "Anonymous";
        }
        os = System.getProperty("os.name")+" (Java)";
    }

    private long uptime() {
        return (System.currentTimeMillis()-created)/1000;
    }
    private String time() {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return dateFormat.format(new Date());
    }
    String buildRequest(String agentType, String aver, String ip, String id, String inner) {
        //noinspection RedundantArrayCreation
        return MessageFormat.format("<iaop pver=\"{0}\" atype=\"{1}\" aver=\"{2}\" os=\"{3}\" " +
                             "ip=\"{4}\" host=\"{5}\" id=\"{6}\" " +
                             "time=\"{7}\" uptime=\"{8}\" iter=\"{9}\">{10}</iaop>",
                             new Object[]{ pver, agentType, aver, os, ip, host, id, time(),
                                           Long.toString(uptime()), Integer.toString(iter), inner});
    }
    Response request(JIAO jiao, String innerRequest) throws JIAOException, IOException {

        if (jiao.id==null) {
            throw new JIAOException("Agent id not specified and register() not called");
        }

        URLConnection connection = new URL(
                System.getenv("DEVENV")!=null?"http://ruondev:8080/iaop1":url).openConnection();
        connection.setDoOutput(true);

        OutputStream output = connection.getOutputStream();
        try {
            String reqXML = buildRequest(jiao.agentType, jiao.agentVersion, ip, jiao.id, innerRequest);
            Writer writer = new OutputStreamWriter(output);
            writer.write(reqXML);
            writer.flush();
        } finally {
            output.close();
        }

        InputStream input = new BufferedInputStream(connection.getInputStream());
        try{
            StringBuffer sb = new StringBuffer();
            int b;
            while ( (b=input.read())>=0) {
                sb.append((char)b);
            }
            Response response = new Response(sb.toString());
            if (response.directive()!=null) {
                iter++;
            }
            return response;
        } finally {
            input.close();
        }
    }
}